{-----------------------------------------------------------------------------}
{ TALSTipDlg v1.0                                                             }
{-----------------------------------------------------------------------------}
{ The ALSTipDlg component allows you to display a Tip of the Day dialog.      }
{ Most of the elements of the dialog can be customized. The tips can be       }
{ stored at design-time, or read from an INI file or the Windows registry     }
{ (available on 32-bit Delphi versions only) at run-time. All you must do     }
{ if fill in the tips at design time (or specify their location) and call     }
{ the component's Execute method.                                             }
{-----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions  }
{ at alazd@mailcity.com                                                       }
{ The lateset version will always be available on the web at:                 }
{   http://members.tripod.com/~flanker27bg/                                   }
{-----------------------------------------------------------------------------}
{ Date last modified:  November 19, 1998                                      }
{-----------------------------------------------------------------------------}

unit ALSTDlg;

interface

uses
  {$IFDEF WIN32}Windows,{$ELSE}WinTypes, WinProcs,{$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms,
  DsgnIntf, Dialogs, INIFiles{$IFDEF WIN32}, Registry{$ENDIF};

type
  TALSShowPos = (posScreenCenter, posFormCenter);
  TTipLocation = (locEmbeded, {$IFDEF WIN32}locRegistry,{$ENDIF} locINIFile);
  TRegRoot = (CURRENT_USER,LOCAL_MACHINE,CURRENT_CONFIG);

  TTipDialogEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb(Index : Integer); override;
    function GetVerb(Index : Integer): string; override;
    function GetVerbCount : Integer; override;
    procedure Edit; override;
  end;

  TAbout = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  TFileNameProperty = class (TStringProperty)
  public
      function getattributes: TPropertyattributes; override;
      procedure Edit; override;
  end;


  TALSTipDlg = class(TComponent)
  private
    FAbout        : TAbout;
    FDlgCaption   : string;
    FTipTitle     : string;
    FTips         : TStrings;
    FTipTitleFont : TFont;
    FTipFont      : TFont;
    FRegRoot      : TRegRoot;
    FRegKey       : string;
    FINIFile      : string;
    FINISection   : string;
    FTipsLocation : TTipLocation;
    FPanelColor   : TColor;
    FCRLFSubst    : string;
    FRandomTips   : boolean;
    FShowAtStartup: boolean;
    FPosition     : TALSShowPos;
  protected
    {$IFDEF WIN32}
    function ReadFromRegistry : boolean;
    {$ENDIF}
    function ReadFromINIFile : boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetTipValues(Value: TStrings);
    procedure SetTitleFont(Value: TFont);
    procedure SetTipFont(Value: TFont);
    function Execute : boolean;
  published
    property About: TAbout read FAbout write FAbout;
    property DlgCaption: string read FDlgCaption write FDlgCaption;
    property TipTitle: string read FTipTitle write FTipTitle;
    property Tips: TStrings read FTips write SetTipValues;
    property TipTitleFont: TFont read FTipTitleFont write SetTitleFont;
    property TipFont: TFont read FTipFont write SetTipFont;
    {$IFDEF WIN32}
    property RegistryRoot: TRegRoot read FRegRoot write FRegRoot default CURRENT_USER;
    property RegistryKey: string read FRegKey write FRegKey;
    {$ENDIF}
    property INIFile: string read FINIFile write FINIFile;
    property INISection: string read FINISection write FINISection;
    property TipsLocation: TTipLocation read FTipsLocation write FTipsLocation default locEmbeded;
    property PanelColor: TColor read FPanelColor write FPanelColor default
                         {$IFDEF VER120}clInfoBk {$ELSE} TColor($00E1FFFF){$ENDIF};
    property CRLFSubstitute : string read FCRLFSubst write FCRLFSubst;
    property RandomTips: boolean read FRandomTips write FRandomTips default False;
    property ShowAtStartup: boolean read FShowAtStartup write FShowAtStartup default True;
    property Position: TALSShowPos read FPosition write FPosition;
  end;

procedure Register;

implementation

uses TipFrm;
{$IFDEF WIN32}
  {$R ALSTDLG.R32}
{$ELSE}
  {$R ALSTDLG.R16}
{$ENDIF}

procedure Register;
begin
  RegisterComponents('Samples', [TALSTipDlg]);
  RegisterComponentEditor(TALSTipDlg, TTipDialogEditor);
  RegisterPropertyEditor(TypeInfo(TAbout), TALSTipDlg, 'ABOUT', TAbout);
  RegisterPropertyEditor(Typeinfo(string), TALSTipDlg, 'INIFile', TFileNameProperty);
end;

procedure DrawTransparent(t: TCanvas; x,y: Integer; s: TBitmap; TrCol: TColor);
var
  bmpXOR, bmpAND, bmpINVAND, bmpTarget: TBitmap;
  oldcol: Longint;
begin
  try
   bmpAND := TBitmap.Create; bmpAND.Width := s.Width; bmpAND.Height := s.Height; bmpAND.Monochrome := True;
   oldcol := SetBkColor(s.Canvas.Handle, ColorToRGB(TrCol));
   BitBlt(bmpAND.Canvas.Handle, 0,0,s.Width,s.Height, s.Canvas.Handle, 0,0, SRCCOPY);
   SetBkColor(s.Canvas.Handle, oldcol);

   bmpINVAND := TBitmap.Create; bmpINVAND.Width := s.Width; bmpINVAND.Height := s.Height; bmpINVAND.Monochrome := True;
   BitBlt(bmpINVAND.Canvas.Handle, 0,0,s.Width,s.Height, bmpAND.Canvas.Handle, 0,0, NOTSRCCOPY);

   bmpXOR := TBitmap.Create; bmpXOR.Width := s.Width; bmpXOR.Height := s.Height;
   BitBlt(bmpXOR.Canvas.Handle, 0,0,s.Width,s.Height, s.Canvas.Handle, 0,0, SRCCOPY);
   BitBlt(bmpXOR.Canvas.Handle, 0,0,s.Width,s.Height, bmpINVAND.Canvas.Handle, 0,0, SRCAND);

   bmpTarget := TBitmap.Create; bmpTarget.Width := s.Width; bmpTarget.Height := s.Height;
   BitBlt(bmpTarget.Canvas.Handle, 0,0,s.Width,s.Height, t.Handle, x,y, SRCCOPY);
   BitBlt(bmpTarget.Canvas.Handle, 0,0,s.Width,s.Height, bmpAND.Canvas.Handle, 0,0, SRCAND);
   BitBlt(bmpTarget.Canvas.Handle, 0,0,s.Width,s.Height, bmpXOR.Canvas.Handle, 0,0, SRCINVERT);
   BitBlt(t.Handle, x,y,s.Width,s.Height, bmpTarget.Canvas.Handle, 0,0, SRCCOPY);
  finally
   bmpXOR.Free;
   bmpAND.Free;
   bmpINVAND.Free;
   bmpTarget.Free;
  end;
end;



{////////////////////////////////////////////////////////////////////////////}
{                                                                            }
{ Dialog design-time preview support                                         }
{                                                                            }
{////////////////////////////////////////////////////////////////////////////}

procedure TTipDialogEditor.ExecuteVerb(Index: Integer);
begin
  if Index <> 0 then Exit;
  Edit;
end;

function TTipDialogEditor.GetVerb(Index: Integer): AnsiString;
begin
  Result := 'Test Tips Dialog';
end;

function TTipDialogEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

procedure TTipDialogEditor.Edit;
begin
   TALSTipDlg(Component).Execute;
end;


{////////////////////////////////////////////////////////////////////////////}
{                                                                            }
{ ABOUT property editor definition                                           }
{                                                                            }
{////////////////////////////////////////////////////////////////////////////}

procedure TAbout.Edit;
begin
  Application.MessageBox('ALS Tip of the Day Dialog component v1.00 for Delphi 1,2,3,4 '+ #13#10+
                         '(C) 1998 Andrew Lazarov.' + #13#10 +
                         'This component is freeware.',
                         'About TALSTipDlg Component', MB_OK + MB_ICONINFORMATION);
end;

function TAbout.GetAttributes: TPropertyAttributes;
begin
  Result:= [paMultiSelect, paDialog, paReadOnly];
end;

function TAbout.GetValue: string;
begin
  Result:= '(about)';
end;


{////////////////////////////////////////////////////////////////////////////}
{                                                                            }
{ INIFile property editor definition                                         }
{                                                                            }
{////////////////////////////////////////////////////////////////////////////}

function TFileNameProperty.GetAttributes;
begin
  Result := [paDialog];
end;

procedure TFilenameProperty.edit;
var
  MFileOpen: TOpenDialog;
begin
  MFileOpen := TOpenDialog.Create(Application);
  MFileOpen.Filename := GetValue;
  MFileOpen.Filter := 'All files|*.*'; 
  MFileOpen.Options := MFileOpen.Options + [ofPathMustExist,ofFileMustExist];
  try
    if MFileOpen.Execute then SetValue(MFileOpen.Filename);
  finally
    MFileOpen.Free;
  end;
end;


{////////////////////////////////////////////////////////////////////////////}
{                                                                            }
{ TALSTipDlg implementation                                                  }
{                                                                            }
{////////////////////////////////////////////////////////////////////////////}

constructor TALSTipDlg.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FTips := TStringList.Create;
     FTipFont := TFont.Create;
     FTipTitleFont := TFont.Create;
     FPanelColor := {$IFDEF VER120} clInfoBk {$ELSE} TColor($00E1FFFF){$ENDIF};
     FRandomTips := False;
     FTipsLocation := locEmbeded;
     FRegRoot := CURRENT_USER;
     FShowAtStartup := True;
end;

destructor TALSTipDlg.Destroy;
begin
     FTips.Free;
     inherited Destroy;
end;

function TALSTipDlg.Execute : boolean;
var
   ADlg : TTipsForm;
   BMP : TBitmap;
begin
     Result := False;
     ADlg := TTipsForm.Create(Application);
     try
        ADlg.TipPanel.Color := FPanelColor;
        BMP := TBitmap.Create;
        BMP.ReleaseHandle;
        BMP.Handle := LoadBitmap(HInstance,'TIPBMP');
        ADlg.Image1.Picture.Bitmap.Width := BMP.Width;
        ADlg.Image1.Picture.Bitmap.Height := BMP.Height;
        ADlg.Image1.Picture.Bitmap.Canvas.Brush.Color := FPanelColor;
        ADlg.Image1.Picture.Bitmap.Canvas.FillRect(Rect(0,0,BMP.Width,BMP.Height));
        DrawTransparent(ADlg.Image1.Picture.Bitmap.Canvas,0,0,BMP,BMP.TransparentColor);

        BMP.Free;
        if FDlgCaption <> '' then ADlg.Caption := FDlgCaption
                             else ADlg.Caption := 'Tip of the day';
        case FTipsLocation of
             locINIFile : begin
                               FTips.Clear;
                               ReadFromINIFile;
                          end;
             {$IFDEF WIN32}
             locRegistry : begin
                               FTips.Clear;
                               ReadFromRegistry;
                           end;
             {$ENDIF}
        end;
        if FTipTitle <> '' then ADlg.TipTitleLbl.Caption := FTipTitle
                           else ADlg.TipTitleLbl.Caption := 'Did you know...';
        ADlg.ShowChk.Checked := FShowAtStartup;
        ADlg.CRLF := FCRLFSubst;
        ADlg.TipLbl.Font.Assign(FTipFont);
        ADlg.TipTitleLbl.Font.Assign(FTipTitleFont);
        ADlg.Tips.Assign(FTips);
        ADlg.IsRandom := FRandomTips;
        ADlg.PrevBtn.Visible := not FRandomTips;
        if FTips.Count > 0 then begin
           if FRandomTips then begin
              Randomize;
              ADlg.CurrentTip := Integer(Random(FTips.Count-1));
           end
           else ADlg.CurrentTip := -1;
           ADlg.NextBtnClick(ADlg.NextBtn);
        end
        else begin
           ADlg.TipLbl.Caption := '';
           ADlg.NextBtn.Enabled := False;
           ADlg.PrevBtn.Enabled := False;
        end;

        case FPosition of
             posScreenCenter : ADlg.Position := poScreenCenter;
             posFormCenter : begin
                if csDesigning in ComponentState then
                   ADlg.Position := poScreenCenter
                else begin
                   ADlg.Top := Application.MainForm.Top +
                               (Application.MainForm.Height div 2) -
                               (ADlg.Height div 2);
                   ADlg.Left := Application.MainForm.Left +
                               (Application.MainForm.Width div 2) -
                               (ADlg.Width div 2);
                end;
             end;
        end;

        ADlg.ShowModal;
        ShowAtStartup := ADlg.ShowChk.Checked;
        Result := ShowAtStartup;
     finally
        ADlg.Free;
     end;
end;

{$IFDEF WIN32}
function TALSTipDlg.ReadFromRegistry : boolean;
var
   Reg : TRegistry;
   ValList : TStringList;
   i : integer;
begin
     Result := False;
     Reg := TRegistry.Create;
     case FRegRoot of
         CURRENT_USER : Reg.RootKey := HKEY_CURRENT_USER;
         LOCAL_MACHINE : Reg.RootKey := HKEY_LOCAL_MACHINE;
         CURRENT_CONFIG : Reg.RootKey := HKEY_CURRENT_CONFIG;
     end;
     if Reg.KeyExists(FRegKey) then begin
        try
           ValList := TStringList.Create;
           Reg.OpenKey(FRegKey,False);
           Reg.GetValueNames(ValList);
           for i := 0 to ValList.Count-1 do begin
               if Reg.GetDataType(ValList[i])=rdString then
                  FTips.Add(Reg.ReadString(ValList[i]));
           end;
           Reg.CloseKey;
           Result := FTips.Count > 0;
        finally
           ValList.Free;
           Reg.Free;
        end;
     end;
end;
{$ENDIF}

function TALSTipDlg.ReadFromINIFile : boolean;
var
   IFile : TINIFile;
   ValList : TStringList;
   i : integer;
begin
     Result := False;
     if (String(FINIFile) = '') or (FINISection = '') then Exit;
     if not FileExists(String(FINIFile)) then Exit;
     IFile := TINIFile.Create(String(FINIFile));
     try
        ValList := TStringList.Create;
        IFile.ReadSection(FINISection, ValList);
        for i := 0 to ValList.Count-1 do
            FTips.Add(IFile.ReadString(FINISection,ValList[i],''));
        Result := True;
     finally
        IFile.Free;
        ValList.Free;
     end;
end;

procedure TALSTipDlg.SetTipFont(Value: TFont);
begin
   FTipFont.Assign(value);
end;

procedure TALSTipDlg.SetTipValues(Value: TStrings);
begin
    Tips.Assign(Value);
end;

procedure TALSTipDlg.SetTitleFont(Value: TFont);
begin
    FTipTitleFont.Assign(value);
end;

end.
